@@ -92,6 +92,10 @@ setMethod("ranef", "unmarkedFitOccu",
 })
 
 
+setMethod("ranef", "unmarkedFitOccuFP", function(object, na.rm = FALSE)
+{
+  cat("ranef is not implemented for occuFP at this time")
+})
 
 
 
@@ -268,7 +272,6 @@ setMethod("ranef", "unmarkedFitGMMorGDS",
     if(is.null(Xlam.offset)) Xlam.offset <- rep(0, nrow(Xlam))
     if(is.null(Xphi.offset)) Xphi.offset <- rep(0, nrow(Xphi))
     if(is.null(Xdet.offset)) Xdet.offset <- rep(0, nrow(Xdet))
-
     beta.lam <- coef(object, type="lambda")
     beta.phi <- coef(object, type="phi")
     beta.det <- coef(object, type="det")
@@ -341,6 +344,7 @@ setMethod("ranef", "unmarkedFitGMMorGDS",
     for(i in 1:nSites) {
         switch(mix,
                P  = f <- dpois(M, lambda[i]),
+               # FIXME: Add ZIP
                NB = f <- dnbinom(M, mu=lambda[i], size=alpha))
         g <- rep(1, K+1) # outside t loop
         for(t in 1:T) {
@@ -372,6 +376,93 @@ setMethod("ranef", "unmarkedFitGMMorGDS",
 
 
 
+setMethod("ranef", "unmarkedFitGPC",
+    function(object, ...)
+{
+    data <- object@data
+    D <- unmarked:::getDesign(data, object@formula)
+
+    Xlam <- D$Xlam
+    Xphi <- D$Xphi
+    Xdet <- D$Xdet
+    y <- D$y  # MxJT
+    Xlam.offset <- D$Xlam.offset
+    Xphi.offset <- D$Xphi.offset
+    Xdet.offset <- D$Xdet.offset
+    if(is.null(Xlam.offset)) Xlam.offset <- rep(0, nrow(Xlam))
+    if(is.null(Xphi.offset)) Xphi.offset <- rep(0, nrow(Xphi))
+    if(is.null(Xdet.offset)) Xdet.offset <- rep(0, nrow(Xdet))
+
+    beta.lam <- coef(object, type="lambda")
+    beta.phi <- coef(object, type="phi")
+    beta.det <- coef(object, type="det")
+
+    R <- nrow(y)
+    T <- data@numPrimary
+    J <- ncol(y) / T
+
+    lambda <- exp(Xlam %*% beta.lam + Xlam.offset)
+    if(is.null(beta.phi))
+        phi <- rep(1, nrow(Xphi))
+    else
+        phi <- plogis(Xphi %*% beta.phi + Xphi.offset)
+    phi <- matrix(phi, R, byrow=TRUE)
+
+    p <- getP(object)
+    p[is.na(y)] <- NA
+    pa <- array(p, c(R,J,T))
+    ya <- array(y, c(R,J,T))
+
+    K <- object@K
+    M <- N <- 0:K
+    lM <- K+1
+
+    post <- array(0, c(R, K+1, 1))
+    colnames(post) <- M
+    mix <- object@mixture
+    if(identical(mix, "NB"))
+        alpha <- exp(coef(object, type="alpha"))
+    for(i in 1:R) {
+        switch(mix,
+               P  = f <- dpois(M, lambda[i]),
+               # FIXME: Add ZIP
+               NB = f <- dnbinom(M, mu=lambda[i], size=alpha))
+        ghi <- rep(0, lM)
+        for(t in 1:T) {
+            gh <- matrix(-Inf, lM, lM)
+            for(m in M) {
+                if(m < max(ya[i,,], na.rm=TRUE)) {
+                    gh[,m+1] <- -Inf
+                    next
+                }
+                if(is.na(phi[i,t])) {
+                    g <- rep(0, lM)
+                    g[N>m] <- -Inf
+                }
+                else
+                    g <- dbinom(N, m, phi[i,t], log=TRUE)
+                h <- rep(0, lM)
+                for(j in 1:J) {
+                    if(is.na(ya[i,j,t]) | is.na(pa[i,j,t]))
+                        next
+                    h <- h + dbinom(ya[i,j,t], N, pa[i,j,t], log=TRUE)
+                }
+                gh[,m+1] <- g + h
+            }
+            ghi <- ghi + log(colSums(exp(gh)))
+        }
+        fgh <- exp(f + ghi)
+        prM <- fgh/sum(fgh)
+        post[i,,1] <- prM
+    }
+    new("unmarkedRanef", post=post)
+})
+
+
+
+
+
+
 setMethod("ranef", "unmarkedFitColExt",
     function(object, ...)
 {
